
import numpy as np
import h5py
from uedge import bbb



def hdf5_restore(file):
    """
        Read a hdf5 file previously written from Uedge. This reads the file and puts
        the 6 standard variables into the correct format.
    """
    try:
        hf = h5py.File(file,'r')
    except:
        print "Couldn't open hdf5 file ",file
        return


    try:
       dummy = hf['bbb']   # force an exception if the group not there
       hfg = hf.get('bbb')
       print "New style hdf5 file"
       try:
           bbb.ngs[...]  = np.array(hfg.get('ngs'))
       except:
           print "Couldn't read ngs from  ",file
       try:
           bbb.nis[...]  = np.array(hfg.get('nis'))
       except:
           print "Couldn't read nis from  ",file
       try:
           bbb.phis[...]  = np.array(hfg.get('phis'))
       except:
           print "Couldn't read phis from  ",file
       try:
           bbb.tes[...]  = np.array(hfg.get('tes'))
       except:
           print "Couldn't read tes from  ",file
       try:
           bbb.tis[...]  = np.array(hfg.get('tis'))
       except:
           print "Couldn't read tis from  ",file
       try:
           bbb.ups[...]  = np.array(hfg.get('ups'))
       except:
           print "Couldn't read ups from  ",file
       try:
           bbb.tgs[...]  = np.array(hfg.get('tgs'))
       except:
           print "Couldn't read tgs from  ",file

    except:
       print "Old style hdf5 file"
       try:
           bbb.ngs[...]  = np.array(hf.get('ngs@bbb'))
       except:
           print "Couldn't read ngs from  ",file
       try:
           bbb.nis[...]  = np.array(hf.get('nis@bbb'))
       except:
           print "Couldn't read nis from  ",file
       try:
           bbb.phis[...]  = np.array(hf.get('phis@bbb'))
       except:
           print "Couldn't read phis from  ",file
       try:
           bbb.tes[...]  = np.array(hf.get('tes@bbb'))
       except:
           print "Couldn't read tes from  ",file
       try:
           bbb.tis[...]  = np.array(hf.get('tis@bbb'))
       except:
           print "Couldn't read tis from  ",file
       try:
           bbb.ups[...]  = np.array(hf.get('ups@bbb'))
       except:
           print "Couldn't read ups from  ",file
       try:
           bbb.tgs[...]  = np.array(hf.get('tgs@bbb'))
       except:
           print "Couldn't read tgs from  ",file

    hf.close()






def hdf5_save(file):
    """
        Write the 6 standard variables into an hdf5 file.
    """
    try:
        hf = h5py.File(file,'w')
        hfg = hf.create_group('bbb')
    except:
        print "Couldn't open hdf5 file ",file
    try:
        hfg.create_dataset('ngs',data=bbb.ngs)
    except:
        print "Couldn't write ngs to  ",file
    try:
        hfg.create_dataset('nis',data=bbb.nis)
    except:
        print "Couldn't write nis to  ",file
    try:
        hfg.create_dataset('phis',data=bbb.phis)
    except:
        print "Couldn't write phis to  ",file
    try:
        hfg.create_dataset('tes',data=bbb.tes)
    except:
        print "Couldn't write tes to  ",file
    try:
        hfg.create_dataset('tis',data=bbb.tis)
    except:
        print "Couldn't write tis to  ",file
    try:
        hfg.create_dataset('ups',data=bbb.ups)
    except:
        print "Couldn't write ups to  ",file
    try:
        hfg.create_dataset('tgs',data=bbb.tgs)
    except:
        print "Couldn't write ups to  ",file

    hf.close()





#
# Python file with some parallel operations
#
parallel_version = "$Id: parallel.py,v 7.0 2018/02/28 18:43:48 meyer8 Exp $"

from Numeric import *
from types import *
# --- Try import mpi - if not found, then run in serial mode
try:
  import mpi
  me = mpi.rank
  npes = mpi.procs
except ImportError:
  me = 0
  npes = 0

if npes > 0: lparallel = 1
else:        lparallel = 0

# --- The interface has changed some in the newest version of pyMPI.
# --- Check the interface to the mpi.recv command. The newer versions
# --- return a tuple instead of just the data itself.
# --- Is there a better way of doing this?
if lparallel:
  mpi.send(me,me)
  _i = mpi.recv(me)
  if type(_i) == TupleType: _newpympi = 1
  else:                     _newpympi = 0
else:
  _newpympi = 1

if _newpympi:
  def mpirecv(pe=0,ms=0):
    result,stat = mpi.recv(pe,ms)
    return result
else:
  def mpirecv(pe=0,ms=0):
    return mpi.recv(pe,ms)

# ---------------------------------------------------------------------------
# --- By default, set so that only PE0 sends output to the terminal.
if lparallel:
  mpi.synchronizeQueuedOutput('/dev/null')

def number_of_PE():
  if not lparallel: return 0
  return mpi.procs
def get_rank():
  if not lparallel: return 0
  return mpi.rank

# ---------------------------------------------------------------------------
# Enable output from all processors
def EnableAll():
  if not lparallel: return
  mpi.synchronizeQueuedOutput(None)

# ---------------------------------------------------------------------------
# Disable output from all but processor 0
def DisableAll():
  if not lparallel: return
  mpi.synchronizeQueuedOutput('/dev/null')

# ---------------------------------------------------------------------------
# Print object on all processors
def pprint(obj):
  if not lparallel:
    print str(obj)
    return
  # Ignore all exceptions to make sure that there is not a lock up.
  try:
    ss = str(obj)
  except:
    ss = ''
  mpi.synchronizedWrite(ss+'\n')
  if mpi.rank == 0: print

# ---------------------------------------------------------------------------
# Print array (or list) from all processors
def aprint(obj):
  if not lparallel:
    print str(obj)
    return
  mpi.synchronizedWrite(str(obj))

# ---------------------------------------------------------------------------
# Get address of processor
def self_address():
  if not lparallel: return 0
  return mpi.rank

# ---------------------------------------------------------------------------
# Copy an array from processor i to processor 0
def getarray(src,v,dest=0):
  if not lparallel: return v
  if mpi.rank == src:
    mpi.send(v,dest)
  elif mpi.rank == dest:
    return mpirecv(src)
  return v

# ---------------------------------------------------------------------------
# Gather an object from all processors into a list
def gather(obj,dest=0):
  if not lparallel: return [obj]
  if mpi.rank == dest:
    result = []
    for i in range(mpi.procs):
      if i == dest:
        result.append(obj)
      else:
        result.append(mpirecv(i))
    return result
  else:
    mpi.send(obj,dest)
    return [obj]

# ---------------------------------------------------------------------------
# Define a barrier
def barrier():
  if not lparallel: return
  mpi.barrier()

# ---------------------------------------------------------------------------
# Broadcast an object to all procerssors
def broadcast(obj,root=0):
  if not lparallel: return obj
  return mpi.bcast(obj,root)

# ---------------------------------------------------------------------------
# Gather an object from all processors into a list and scatter back to all
def gatherall(obj):
  if not lparallel: return obj
  obj = gather(obj)
  return mpi.bcast(obj)
    
# ---------------------------------------------------------------------------
# General gatherarray which returns an array object combining the
# first dimension.
def gatherarray(a,root=0,othersempty=0,bcast=0):
  if not lparallel: return a
  # --- First check if input can be converted to an array
  isinputok = 1
  try:
    if type(a) in [type(0.),type(0)]:
      a = array([a])
    else:
      a = array(a)
  except:
    isinputok = 0
  # --- Make sure the input is ok on all of the processors
  isinputok = globalmin(isinputok)
  # --- If any returned an error, then all exit (to avoid a deadlock)
  if not isinputok:
    print "Object could not be converted to an array"
    return None
  # --- Now, actually gather the array.
  # --- The check of whether the result is ok may not be needed.
  try:
    result = gather(a,root)
    isinputok = 1
  except:
    isinputok = 0
  # --- Make sure again that the input is ok on all of the processors
  isinputok = globalmin(isinputok)
  if not isinputok:
    print "Error in gather object"
    if type(a) == ArrayType: print "Object has shape ",shape(a)
    return None
  # --- All processors but root simply return either the input argument
  # --- or an empty array unless the result is to be broadcast
  if me != root and not bcast:
    if othersempty: return zeros(len(shape(a))*[0],a.typecode())
    else: return a
  # --- Root processor reshapes the data, removing the first dimension
  # --- Do it bit by bit since the data passed by the other processors may
  # --- not be all the same size.
  if me == root:
    newlen = 0
    for i in range(npes):
      newlen = newlen + shape(result[i])[0]
    newshape = list(shape(result[0]))
    newshape[0] = newlen
    newresult = zeros(newshape,a.typecode())
    i1 = 0
    for i in range(npes):
      i2 = i1 + shape(result[i])[0]
      newresult[i1:i2,...] = result[i]
      i1 = i2
  else:
    newresult = 0
  if bcast: newresult = mpi.bcast(newresult,root)
  return newresult
  ## --- Old way
  ## --- Its easy if all of the arrays passed from the other processors
  ## --- are the same size.
  #result = array(result)
  #ss = list(shape(result))
  #snew = ss[1:]
  #snew[0] = ss[0]*ss[1]
  #result.shape = snew
  ## --- Return the result
  #return result


# ---------------------------------------------------------------------------
# Find the nonzero value of array over all processors. This assumes that the
# non-zero values for each index are the same for all processors.
# Resulting data is broadcast to all processors.
def parallelnonzeroarray(a):
  dmax = parallelmax(a)
  dmin = parallelmin(a)
  result = where(not_equal(dmax,0),dmax,dmin)
  return result

# ---------------------------------------------------------------------------
# Generic global operation on a distributed array.
def globalop(a,localop,mpiop,defaultval):
  if type(a) in [FloatType,IntType]:
    local = a
  elif len(a) > 0:
    try:
      # --- Protect application of localop since the data may not be
      # --- appropriate on all processors, for example it may be an empty array.
      local = localop(a)
    except:
      local = defaultval
  else:
    local = defaultval
  if not lparallel: return local
  return mpi.allreduce(local,getattr(mpi,mpiop))

# ---------------------------------------------------------------------------
# Specific operations on a distributed array.
def globalmax(a):
  return globalop(a,max,"MAX",-1.e36)
def globalmin(a):
  return globalop(a,min,"MIN",+1.e36)
def globalsum(a):
  return globalop(a,sum,"SUM",0.)
def globalave(a):
  s = globalop(a,sum,"SUM",0.)
  if type(a) in [FloatType,IntType]: a = [a]
  n = globalsum(len(a))
  if n > 0: return s/n
  else:     return 0.

# ---------------------------------------------------------------------------
# Generic parallel element-by-element operation on a distributed array.
def parallelop(a,mpiop):
  if not lparallel: return a
  if type(a) == type(array([])):
    a1d = ravel(a) + 0
    for i in range(len(a1d)):
      a1d[i] = mpi.allreduce(a1d[i],getattr(mpi,mpiop))
    a1d.shape = shape(a)
    return a1d
  else:
    return mpi.allreduce(a,getattr(mpi,mpiop))


# ---------------------------------------------------------------------------
# Specific parallel element-by-element operations on a distributed array.
def parallelmax(a):
  return parallelop(a,"MAX")
def parallelmin(a):
  return parallelop(a,"MIN")
def parallelsum(a):
  if not lparallel: return a
  return mpi.allreduce(a,mpi.SUM)





#!/bin/sh

#from matplotlib import pylab
import matplotlib.pyplot as plt
import numpy
import tables
import sys

f=tables.openFile("2200_edge_1.h5")
#f=tables.openFile("2200_edge_10.h5")
vsf=tables.openFile("2200_coupled-lineout_9.vsh5")

# Electron temperature
tes=f.root.bbb.tesCore.read()[:,1:]
tesCoreAve=numpy.average(tes,0)
tesCoreStd=numpy.std(tes,0)
maxTeLims=tes.max()              #for figure limits
#tes=f.root.bbb.tesSol.read()
tes=vsf.root.outer_temperature_electron.read()
tesSolAve=tes
tesSolStd=0.

# Ion temperature
tis=f.root.bbb.tisCore.read()[:,1:]
tisCoreAve=numpy.average(tis,0)
tisCoreStd=numpy.std(tis,0)
maxTiLims=tis.max()
#tis=f.root.bbb.tisSol.read()
tis=vsf.root.outer_temperature_H2p1.read()
tisSolAve=tis
tisSolStd=0.

# Density
nis=f.root.bbb.nisCore.read()[:,1:,0]/2.e4
nisCoreAve=numpy.average(nis,0)
nisCoreStd=numpy.std(nis,0)
nis=vsf.root.outer_density.read()/2.e4
#nis=f.root.bbb.nisSol.read()[:,:,0]/2.e4
nisSolAve=nis
nisSolStd=0.

# Neutral gas
ngs=f.root.bbb.ngsCore.read()[:,1:,0]
ngsCoreAve=numpy.average(ngs,0)
ngsCoreStd=numpy.std(ngs,0)
ngs=f.root.bbb.ngsSol.read()
#ngs=f.root.bbb.ngsSol.read()
ngsSolAve=numpy.average(ngs,0)
ngsSolStd=numpy.std(ngs,0)

# Grid
gridCore=vsf.root.mpEdgeGrid.read()[1:]
gridEdge=vsf.root.mpOuterGrid.read()

maxTlim=numpy.maximum(maxTeLims,maxTiLims)*1.1

# Now switch to a more OO interface to exercise more features.
fig, axs = plt.subplots(nrows=2, ncols=1)
ax = axs[0]
ax.errorbar(gridCore, tesCoreAve, yerr=tesCoreStd, label="Electron temperature")
ax.errorbar(gridCore, tisCoreAve, yerr=tisCoreStd, label="Ion temperature")
#ax.errorbar(gridEdge, tesSolAve, yerr=tesSolStd, label="Electron temperature")
#ax.errorbar(gridEdge, tisSolAve, yerr=tisSolStd, label="Ion temperature")
ax.set_title('Average Temperatures')
ax.set_ylim(0,maxTlim)
ax.legend(loc='lower left',frameon=False)

ax = axs[1]
ax.errorbar(gridCore, nisCoreAve, yerr=nisCoreStd, label="Ion density/2.e4")
ax.errorbar(gridCore, ngsCoreAve, yerr=ngsCoreStd, label="Neutral gas density")
#ax.errorbar(gridEdge, nisSolAve, yerr=nisSolStd, label="Ion density")
#ax.errorbar(gridEdge, ngsSolAve, yerr=0., label="Neutral gas density")
ax.set_title('Average Densities')
ax.legend(loc='upper left',frameon=False)

fig.suptitle('Average quantities showing standard deviation.')

plt.show()



# Methods to create a list of files in a directory, and a sub-list containing sspecified suffixes.

import os
import string
def filelist(path):
   return os.listdir(path)

def filesublist(path,suffix):
   sublist = []
   biglist = filelist(path)
   for filename in biglist:
       filesplit = string.split(filename,".")
       if filesplit[-1] == suffix:
           sublist.append(filename)
   return sublist





"""Utility and convenience routines used in Uedge

iota(): returns a array of sequential integers
remark(): same as print
sign(): emulation of sign function
arrayappend(): appends to multi-dimensional array
exists(): checks if a variable exists
ave(): averages an array of numbers
maxnd(): finds max of multi-dimensional array
minnd(): finds min of multi-dimensional array
getnextfilename(): finds next available file name in a numeric sequence
getmesh2d(): returns tuple of 2 2-d arrays holding the coordinates of the
             two dimensions
getmesh3d(): returns tuple of 3 3-d arrays holding the coordinates of the
             three dimensions
averagezdata(): Does local averaging over the first dimension of the input
                array. Can also do down select on the data.
getdatafromtextfile(): Reads in table data from a text file, returning an array
                       that holds that data.

"""
from __future__ import generators # needed for yield statement for P2.2
from uedge import *

uedgeutils_version = "$Id: uedgeutils.py,v 7.0 2018/02/28 18:43:48 meyer8 Exp $"

def uedgeutilsdoc():
  import uedgeutils
  print uedgeutils.__doc__

# --- Convenience function modeled after the iota of basis
def iota(low,high=None,step=1):
  if high is None:
    if step > 0:
      return arange(1,low+1,step)
    else:
      return arange(low,0,step)
  else:
    if step > 0:
      return arange(low,high+1,step)
    else:
      return arange(low,high-1,step)

# --- Convenience function to do printing
def remark(s):
  print s

# --- Replicate the sign function with two arguments. If only one is
# --- given, return the value from the Numeric sign function.
# --- This should realy be removed.
numericsign = sign
def sign(x,y=None):
  if y is None: return numericsign(x)
  if type(x) == ArrayType:
    result = where(greater(y,0.),abs(x),-abs(x))
    result = where(equal(y,0.),0.,result)
    return result
  else:
    if y > 0:
      return abs(x)
    elif y < 0:
      return -abs(x)
    else:
      return 0

# --- Convenience function which returns meshes filled with the coordinates.
def getmeshcoordinates(mins,dds,nns):
  """
getmeshcoordinates(mins,dds,nns)
Returns arrays holding the coordinates of the mesh points.
Lenght of list of inputs determines number of dimensions.
  """
  nns = tuple(array(nns) + 1)
  cc = indices(nns,'d')
  for i in xrange(len(mins)): cc[i] = mins[i] + cc[i]*dds[i]
  clist = []
  for i in xrange(len(mins)): clist.append(cc[i])
  return tuple(clist)

def getmesh2d(xmin,dx,nx,ymin,dy,ny):
  """
getmesh2d(xmin,dx,nx,ymin,dy,ny)
Returns 2 2-d arrays holding the coordinates of the mesh points
  """
  return getmeshcoordinates([xmin,ymin],[dx,dy],[nx,ny])

def arrayappend(x,a):
  xshape = list(shape(x))
  if type(a) == ArrayType:
    pass
  elif type(a) == ListType:
    a = array(a)
  else:
    a = array([a])
  ashape = list(shape(a))
  if len(xshape)==1 and len(ashape)==1:
    xshape[0] = xshape[0] + ashape[0]
    y = fzeros(xshape,x.typecode())
    y[0:xshape[0]-ashape[0]] = x
    y[xshape[0]-ashape[0]:] = a
  elif len(xshape) == len(ashape)+1 and xshape[:-1] == ashape:
    xshape[-1] = xshape[-1] + 1
    y = fzeros(xshape,x.typecode())
    y[...,0:-1] = x
    y[...,-1] = a
  return y

# Convenience function which returns true if variable exists
def exists(x):
  """
Checks whether or not the variable whose name is specified exists in the
main dictionary.
 - x: Name of variable - must be a string.
  """
  import __main__
  if x in __main__.__dict__.keys(): return true
  if x in locals().keys(): return true
  if x in globals().keys(): return true
  return false

# Returns the average of the input array
def ave(x,index=0):
  if len(shape(x)) == 0: return x
  if shape(x)[index] > 0:
    return sum(x,index)/shape(x)[index]
  else:
    return 0.

def averagezdata(qty, navg=0, nlines=100, n1=None, n2=None, istep=None,
                 includezeros=Forthon.false):
  """
Averages data over local region. It also can down select data in the other
dimension.
  - qty: Data to be smoothed. Can be either a 1-D or 2-D array.
  - navg=0: number of data points to average over
  - nlines=100: number of lines from second dimension to choose.
  - n1=shape(qty)[0]-1:
  - n2=shape(qty)[1]-1:
  - istep=max(1,n2/nlines):
  - includezeros=false: by default, only non-zero data is averaged over.
  """
  if shape(qty)[0] < navg+navg+1: return qty
  if navg == 0 or nlines == 0: return qty
  if len(shape(qty)) == 1:
    fixqty = 1
    qty.shape = (len(qty),1)
  else:
    fixqty = 0
  if not n1: n1 = shape(qty)[0] - 1
  if not n2: n2 = shape(qty)[1] - 1
  if istep is None: istep = max(1,n2/nlines)
  hl = qty[:,::istep] + 0.
  hl[navg,:] = sum(qty[navg-navg:navg+navg+1,::istep])
  nn = 2*navg+1 + zeros(shape(hl))
  if not includezeros:
    nn[navg,:] = sum(where(qty[navg-navg:navg+navg+1,::istep]==0.,0,1),0)
  for j in range(navg+1,n1-navg-1):
    hl[j,:] = hl[j-1,:] + (qty[j+navg,::istep] - qty[j-navg-1,::istep])
    nn[j,:] = nn[j-1,:] + (+ where(qty[j+navg,::istep]==0,0,1)
                           - where(qty[j-navg-1,::istep]==0,0,1))
  nn = where(nn==0,1,nn)
  hl = where(qty[:,::istep]==0.,0.,hl)
  hl[navg:n1-navg-1,:] = hl[navg:n1-navg-1,:]/nn[navg:n1-navg-1,:]
  if fixqty: qty.shape = (shape(qty)[0],)
  if n2 > 1: return hl
  else: return hl[:,0]

# --- Returns the max of the multiarray
def maxnd(x,defaultval=-1.e36):
  """Return the max element of an array of any dimension"""
  xtemp = reshape(x,tuple([product(array(shape(x)))]))
  if len(xtemp) == 0: return defaultval
  return max(xtemp)
# --- Returns the min of the multiarray
def minnd(x,defaultval=+1.e36):
  """Return the min element of an array of any dimension"""
  xtemp = reshape(x,tuple([product(array(shape(x)))]))
  if len(xtemp) == 0: return defaultval
  return min(xtemp)
# --- Returns the sum of the multiarray
def sumnd(x,defaultval=0.):
  """Return the total sum of an array of any dimension"""
  xtemp = reshape(x,tuple([product(array(shape(x)))]))
  if len(xtemp) == 0: return defaultval
  return sum(xtemp)
# --- Returns the sum of the multiarray
def avend(x,defaultval=0.):
  """Return the average of an array of any dimension"""
  xtemp = reshape(x,tuple([product(array(shape(x)))]))
  if len(xtemp) == 0: return defaultval
  return sum(xtemp)/len(xtemp)

# Gets next available filename with the format 'root.nnn.suffix'.
def getnextfilename(root,suffix):
  dir = string.join(os.listdir('.'))
  i = 0
  name = root+('.%03d.'%i)+suffix
  while re.search(name,dir):
    i = i + 1
    name = root+('.%03d.'%i)+suffix
  return name

# --- Another Example profiler - this prints out the function name when the
# --- function is called and when it returns. Indentation is used to signify
# --- the call level.
# --- To use, execute the command sys.setprofile(uedgeprofile).
def uedgeprofilesample(frame,event,arg):
  try:    uedgeprofile.level
  except: uedgeprofile.level = 0
  if event == 'return': uedgeprofile.level = uedgeprofile.level - 1
  print "%s %s %s"%(uedgeprofile.level*'  ',event,frame.f_code.co_name)
  if event == 'call': uedgeprofile.level = uedgeprofile.level + 1

# --- Create enumerate function, which is defined in python2.3 but not 2.2
try:
  enumerate
except:
  def enumerate(ll):
    tt = []
    for i in range(len(ll)):
      tt.append((i,ll[i]))
    return tt

# --- Convenience function to read in data from a text file
def getdatafromtextfile(filename,nskip=0,dims=[],nquantities=1,typecode='d',
                        fortranordering=1,converter=float):
  """
Reads data in from a text file. The data is assumed to be laid out on a
logically Cartesian mesh.
 - filename: must be supplied
 - nskip=0: numbers of lines at the beginning of the file to skip
            e.g. lines with file info and comments
 - dims=[]: must be supplied - the size of each dimension
 - nquantities=1: number of quantities in the file
 - fortranordering=1: when true, the data will be in fortran ordering, where
                      the index that varies that fastest in the file will be
                      the first index. Otherwise use C ordering.
 - converter=float: Function which converts the strings into numbers. This should
                    be the type of result desired. It should only be float or int,
                    unless you know what you are doing.
  """

  # --- Get total number of data values and make an array big enough to hold
  # --- them.
  ntot = nquantities*product(dims)
  data = zeros(ntot,typecode)

  ff = open(filename,'r')

  # --- Skip the number of lines at the top of the file as specified
  for i in range(nskip):
    ff.readline()

  # --- Loop over the file, reading in one line at a time.
  # --- Each whole line is put into data at once.
  # --- For the conversion of the strings into numbers, it is faster to use
  # --- float or int rather than eval.
  i = 0
  while i < ntot:
    dataline = map(converter,string.split(ff.readline()))
    nd = len(dataline)
    data[i:i+nd] = dataline
    i = i + nd

  if not fortranordering:
    # --- reverse the order of the dims to prepare the transpose that follows
    dims = list(dims)
    dims.reverse()

  # --- If nquantities, then add another dimension to the data array
  if nquantities > 1:
    dims0 = dims
    dims = [nquantities] + list(dims)
  else:
    dims = list(dims)

  # --- Set array to have proper shape.
  dims.reverse()
  data.shape = tuple(dims)
  if fortranordering:
    data = transpose(data)
  else:
    data = transpose(data,[len(dims0)]+range(len(dims0)))

  return data




import os
import contextlib
import unittest

print "in uedge_test"

@contextlib.contextmanager
def pushd(new_dir):
    previous_dir = os.getcwd()
    os.chdir(new_dir)
    try:
       yield 
    finally:
       os.chdir(previous_dir)


try:
  if os.path.exists('pytests/level_1/dotest.py'):
     with pushd('pytests/level_1'):
         execfile('dotest.py')
except:
  pass

try:
  if os.path.exists('pytests/level_2/dotest.py'):
      with pushd('pytests/level_2'):
         execfile('dotest.py')
except:
  pass




#!/usr/bin/env python

# @description This script will convert a pdb data file into an hdf5
#              data file
#
# @keywords    Py-UEDGE
#
# @contact     John Cary
#
# @version     $id: $

# import sys for system commands
import sys, os

# Import uefacets for interface convenience
import uefacets

# import getopts for cmd line parsing
import getopt

# import uedge and all its structures
from uedge import *

def usage(code):
    print "pdb2h5 [-h] -i <infile> -o <outfile> [-p <paramfile>]"
    print " -h: Print this help"
    sys.exit(code)

try:
    olst, _ = getopt.getopt(sys.argv[1:], "hi:o:p:")
except:
    usage(1)

for o in olst:
    if o[0] == "-h":
        usage(0)
    if o[0] == "-i":
        infile = o[1]
    if o[0] == "-o":
        outfile = o[1]
    if o[0] == "-p":
        prmfile = o[1]

# initialize the uedge object

uefacets.init()

ue = uefacets.Uedge()

try:
    ue.readParams(prmfile)
except NameError, _:
    print "No parameter file specified, continuing..."

ue.buildData()
ue.restore(infile)
ue.dump(outfile)

uefacets.final()

sys.exit(0)




#
# $Id: uefacets.py,v 7.0 2018/02/28 18:43:48 meyer8 Exp $
# Implementation of uedge methods for FACETS
#
# Substantial revisions R. Cohen 4/21/08
#
# Caveat 1 for now: We are not distinguishing between parallel and
# toroidal momentum fluxes at the interface.  (This is better than
# previously when the toroidal and parallel velocities themselves
# were treated as indistinguishable.)
# Doing better will require further work to ensure consistency between
# the core and edge models.
# Caveat 2 for now: no coupling of the potential equation.  Closely
# related to caveat 1.

import __main__
import string
import types
from copy import copy
from uedge import *
from uewall import *

joule_per_ev = 1.6022e-19

# Import pytables
try:
#SEK  import tables
  # bbb.writeToLog( "tables imported." )
  # from tables.nodes import FileNode
  # from tables import FileNode
  # bbb.writeToLog( "FileNode imported." )
  import numpy as numerix
  have_tables = 1
except ImportError, inst:
  # bbb.writeToLog( "ImportError:", inst )
  have_tables = 0
except:
  # bbb.writeToLog( "Exception:", sys.exc_info() )
  have_tables = 0

class UedgeError:

  def __init__(self, msg):
    self.msg = msg

  def __str__(self):
    return self.msg

iset_petsc_local=0
iset_mpi_local=0
def init(localPetsc=True, localComm=True):
# Initialize MPI and PETSc
    global iset_petsc_local
    global iset_mpi_local
    flx.init_hdf5()
    if (localComm):
       bbb.uedge_mpiInit()
       iset_mpi_local=1
       # if not bbb.mype:
         # bbb.writeToLog( "NOTE: localComm is True on rank %d.  Should not be a component."%bbb.mype )
    if (localPetsc):
       bbb.uedge_petscInit()
       iset_petsc_local=1
       # if not bbb.mype:
         # bbb.writeToLog( "PETSc initialized on rank %d."%bbb.mype )

def final():
# Finalize petsc and MPI
    global iset_petsc_local
    global iset_mpi_local
    if iset_petsc_local:
       if not bbb.mype:
         bbb.writeToLog( "uedge_petscFinal called on rank %d."%bbb.mype )
       bbb.uedge_petscFinal()
    if iset_mpi_local:
       if not bbb.mype:
         bbb.writeToLog( "uedge_mpiFinal called on rank %d."%bbb.mype )
       bbb.uedge_mpiFinal()

class Uedge:

  def __init__(self):
    """Create an instance of UEDGE that can be executed through Python
    This will register the dictionaries at the top level.
    """
# Initialize the time, step, ...
    self.t = 0.
    self.t_old = 0.
    self.initcall = 1

# create an empty setter function dictionary; will register
# all the set functions into this dict with their proper
# string names
    self.setMap = {}
    self.setMap["nspecies_CE"] = 0 # = self.setNSpecies

# create an empty getter function dictionary; will register
# all the get functions into this dict with their proper
# string names.
    self.getMap = {}

# Get all of the dictionaries registered
    __main__.__dict__['bbb'] = bbb
    __main__.__dict__['aph'] = aph
    __main__.__dict__['com'] = com
    __main__.__dict__['flx'] = flx
    __main__.__dict__['api'] = api
    __main__.__dict__['grd'] = grd
    __main__.__dict__['svr'] = svr
    __main__.__dict__['wdf'] = wdf

# For coupled runs, turn off uedge's core recycling.  Eventually
# we should have a way of telling UEDGE whether we are running
# coupled or not.
    bbb.recycc = 0.

    # Create an instance of the UEDGE wall interface class
    self.uewall = UEwall()

# Set up dictionary of methods
    # density setters and flux setters will be handled with different
    # string signatures.  density_* will be for density (below) and
    # ptclFlux_* will be for fluxes
    # Note RHS is a list [method, atomic number, atomic mass, charge state]
    self.setMap["density_CE_H2p1"]                 = [self.setNi_CE,1,2,1]
    self.setMap["density_CE_H2"]                   = [self.setNg_CE,1,2]
    self.setMap["rotationFrequency_CE_H2p1"]       = [self.setUp_CE,1,2,1]
    self.setMap["temperature_CE_H2p1"]             = [self.setTi_CE,1,2,1]
    self.setMap["temperature_CE_electron"]         = self.setTe_CE
    self.setMap["temperature_CE_H2"]               = [self.setTg_CE,1,2]

    self.setMap["ptclFlux_CE_H2p1"]                = [self.setNiFlux_CE,1,2,1]
    self.setMap["ptclFlux_CE_H2"]                  = [self.setNgFlux_CE,1,2]
    self.setMap["angularMomentumFlux_CE_H2p1"]     = [self.setUpFlux_CE,1,2,1]
    self.setMap["energyFlux_CE_H2p1"]              = [self.setTiFlux_CE,1,2,1]
    self.setMap["energyFlux_CE_electron"]          = self.setTeFlux_CE
    self.setMap["energyFlux_CE_H2"]                = [self.setTgFlux_CE,1,2]

    self.getMap["density_CE_H2"]                   = [self.getNg_CE,1,2]
    self.getMap["density_CE_H2p1"]                 = [self.getNi_CE,1,2,1]
    self.getMap["rotationFrequency_CE_H2p1"]       = [self.getUp_CE,1,2,1]
    self.getMap["temperature_CE_H2p1"]             = [self.getTi_CE,1,2,1]
    self.getMap["temperature_CE_electron"]         = self.getTe_CE
    self.getMap["temperature_CE_H2"]               = [self.getTg_CE,1,2]

    self.getMap["ptclFlux_CE_H2p1"]                = [self.getNiFlux_CE,1,2,1]
    self.getMap["ptclFlux_CE_H2"]                  = [self.getNgFlux_CE,1,2]
    self.getMap["angularMomentumFlux_CE_H2p1"]     = [self.getUpFlux_CE,1,2,1]
    self.getMap["energyFlux_CE_H2p1"]              = [self.getTiFlux_CE,1,2,1]
    self.getMap["energyFlux_CE_electron"]          = self.getTeFlux_CE
    self.getMap["energyFlux_CE_H2"]                = [self.getTgFlux_CE,1,2]

    # getters for various diffuson coefficients
    self.getMap["densityDiffusion_CE_H2p1"]          = [self.getDensDif_CE,1,2,1]
    self.getMap["densityConvection_CE_H2p1"]         = [self.getDensConv_CE,1,2,1]
    self.getMap["parmomentumDiffusion_CE_H2p1"]      = [self.getParmomDif_CE,1,2,1]
    self.getMap["parmomentumConvection_CE_H2p1"]     = [self.getParmomConv_CE,1,2,1]
    self.getMap["temperatureDiffusion_CE_H2p1"]      = [self.getTiDif_CE,1,2,1]
    self.getMap["temperatureConvection_CE_H2p1"]     = [self.getTiConv_CE,1,2,1]
    self.getMap["temperatureDiffusion_CE_electron"]  = self.getTeDif_CE
    self.getMap["temperatureConvection_CE_electron"] = self.getTeConv_CE

    # map entires for provenance data
    self.setMap["dataDirectory"]                     = self.setShareDir

    # add edge-wall interface quantities to get, setMap
    self.uewall.initialize()

  def readParams(self, sourcefile):
    """Read the parameters from the file, sourcefile.
    This file should set parameters only.  It should not call any methods.
    """
    if not bbb.mype:
      # bbb.writeToLog( "Uedge.readParams called with file, %s."%sourcefile )
      pass
    try:
      execfile(sourcefile)
    except:
      raise
    if not bbb.mype:
      bbb.writeToLog( "Uedge.readParams: %s sourced."%sourcefile )
    # if petsc enabled - readin petsc params [options] aswell.
    bbb.uedge_petscInsertOpts()

  def setLogFile(self, logFile):
    bbb.setLogFile(logFile)

  def setComm(self, mpiComm=None):
# Eventually: will set up MPI
    bbb.set_uedgeComm(mpiComm)

  def buildData(self):
    """buildData should now allocate all of the data.
    """
    if not bbb.mype:
      # bbb.writeToLog( "Uedge.buildData: entered." )
      pass
    grd.rplate1 = [ 0 for i in range(0, grd.nplate1) ]
    grd.zplate1 = [ 0 for i in range(0, grd.nplate1) ]
    grd.rplate2 = [ 0 for i in range(0, grd.nplate2) ]
    grd.zplate2 = [ 0 for i in range(0, grd.nplate2) ]

    bbb.allocate()
    if bbb.ismpion == 1:
      bbb.init_par_meshg()
    if not bbb.mype:
      bbb.writeToLog( "Uedge.buildData: allocation complete." )

  def buildUpdaters(self):
    # this constructs the solver itself.  Assumptions about data can be
    # made, since buildData is guaranteed to be called before this.
    # However, no assumptions can be made about the state of the data, as
    # it is not guaranteed that initialize will have been called before
    # this
    pass

  def fluxSurfAv_CE(self,InArray):
    # utility function to take flux surface averages on core-edge interface.
    # InArray is either a 1-D array defined on UEDGE's core poloidal array, of
    # length com.ixpt2-com.ixpt1, or a full 2-D array on UEDGE's full grid

    ixpt11 = com.ixpt1[0]+1; ixpt21 = com.ixpt2[0]+1
    if (len(shape(InArray)) == 1):
        return sum(com.volv[ixpt11:ixpt21,0]*InArray)/   \
               sum(com.vol[ixpt11:ixpt21,0])
    elif (len(shape(InArray)) == 2):
        return sum(com.volv[ixpt11:ixpt21,0]*InArray[ixpt11:ixpt21,0])/   \
          sum(com.vol[ixpt11:ixpt21,0])
    else:
        raise "Improper dimension for InArray"


  # set boundary parameters in the UEDGE object.  Any data processing and
  # massaging that needs to be done should be performed in these
  # functions.

  def gradAv_CE(self,InArray):
    # utility function to take gradient-weighted averages over a flux surface
    # in other words, returns Int(inArray*(d/dr))/Int(d/dr).
    # InArray must be 2D array defined over UEDGE's full grid.
    ixpt11 = com.ixpt1[0]+1; ixpt21 = com.ixpt2[0]+1
    return sum(com.gy[ixpt11:ixpt21,0]*com.sy[ixpt11:ixpt21,0] * \
	InArray[ixpt11:ixpt21,0]) / sum(com.gy[ixpt11:ixpt21,0] * \
	com.sy[ixpt11:ixpt21,0])

  def initialize(self):
    # Initialize the solver and prepare for runs.
    # This assumes that the data has been completely constructed and that the
    # solver is constructed and ready for use.  This is called last in
    # the initialization callpath.
    # Per facets rules, initialize should contain only things that are
    # used for a new run but not a restart.
    #
    self.commonsetup(restart=0)

  def commonsetup(self,restart):
    # Set up mass and charge arrays and flux surface average arrays needed
    # for the variable setting methods.  Must have arrays allocated and so must
    # be called after buildData.  Must know whether this is a restart,
    # to set bbb.restart in order to be able to call this from
    # initialize() or from restore()
    bbb.restart=restart

    if bbb.ismpion and restart == 1:
    # if bbb.ismpion:	# Hangs
      bbb.writeToLog( "WARNING: For parallel run, averages not computed by commonsetup." )
      return

    bbb.ueinit()

    # ion atomic mass and charge state lists
    self.imasslist = bbb.minu.round(0).tolist()
    self.chargelist = bbb.ziin.round(0).tolist()
    # neutral atomic mass list, construct from unique members of imasslist.
    # this will fail (produce run-time error) if com.ngsp is bigger than
    # the length of this list; but it is also a failure of the physics
    # specification.
    self.nmasslist = uniquelist(self.imasslist)

    self.ixpt11 = com.ixpt1 + 1
    self.ixpt21 = com.ixpt2 + 1

    ixpt11 = self.ixpt11
    ixpt21 = self.ixpt21
    self.Raverage = bbb.fluxsurfav1(com.rm[ixpt11:ixpt21,0,0])
    # In above, making use of thin-ness of guard cell so  needn't worry
    #  abut using centered cell value of r.

  def finalize(self):
    # this finalizes the UEDGE object.  Note that this call should NOT
    # destroy data.  Instead, all postprocessing of data and solver state
    # should be performed here.  Data destruction should be left up to
    # the class destructor.
    pass

  def getDouble(self, name):
    # get data used by UEDGE.  This function should stay as is, all
    # implementation should be done in the functions registered in
    # the map.  Note this can be used by UEDGE experts to get any variable
    # in UEDGE's variable descriptor files, if name is of the form
    # packagename.variablename
    self.nisp=com.nisp
    self.ngsp=com.ngsp
    self.nisp1=self.nisp+1
    self.nusp=com.nusp

    try:
      rhs = self.getMap[name]
      if type(rhs)==types.ListType:
        # ions or neutrals
        if len(rhs) == 4:
          # ions
          return rhs[0](self.findindexi(rhs[1:]))
        if len(rhs) == 3:
          # neutrals
          return rhs[0](self.findindexn(rhs[1:]))
      else:
          # electrons
          return rhs()
    except:
      # The expert option.
      print eval(name)
      return eval(name)
      # An exception will be raised if name is invalid

  def setDouble(self, name, value):
    # set data used by UEDGE.  This function should stay as is, all
    # implementation should be done in the functions registered in
    # the map.  Note this can be used by UEDGE experts to set any variable
    # in UEDGE's variable descriptor files, if name is of the form
    # packagename.variablename

    #bbb.writeToLog( "Inside uefacets::setDouble with name ", name, " value ", value )

    try:
      rhs = self.setMap[name]
      if type(rhs)==types.ListType:
        # ions or neutrals
        if len(rhs) == 4:
          rhs[0](value,self.findindexi(rhs[1:]))
        if len(rhs) == 3:
          # neutrals
          rhs[0](value,self.findindexn(rhs[1:]))
      else:
        # electrons
        rhs(value)
    except:
      # The expert option.
      bbb.writeToLog( "Using expert option to set a parameter" )
      bbb.writeToLog( "EXPERT OPTION WITH NAME '%s'" % name )
      exec(name+"="+`value`)
      # An exception will be raised if name is invalid

  def setString(self, name, value):
    # set data used by UEDGE.  This function should stay as is, all
    # implementation should be done in the functions registered in
    # the map.  Note this can be used by UEDGE experts to set any variable
    # in UEDGE's variable descriptor files, if name is of the form
    # packagename.variablename
    print name, value
    # self.setMap[name](value)

  def getString(self, name):
    # get data used by UEDGE.  This function should stay as is, all
    # implementation should be done in the functions registered in
    # the map.  Note this can be used by UEDGE experts to get any variable
    # in UEDGE's variable descriptor files, if name is of the form
    # packagename.variablename
    self.getMap[name](value)

  def getDoubleAtIndex(self, name,ndim,loc_index):
    # get data used by UEDGE.  This function should stay as is, all
    # implementation should be done in the functions registered in
    # the map.  It is assumed that any invocation of getDoubleAtIndex
    # is targeted at quantities for communication with the wall,
    # so name has a "_EW" attached.
    if name.find("_EW") < 0:
      raise name+" is invalid edge-wall interface name"
    self.nisp=com.nisp
    self.ngsp=com.ngsp
    self.nisp1=self.nisp+1
    self.nusp=com.nusp

    rhs = self.uewall.getMap[name]
    if type(rhs)==types.ListType:
      # ions or neutrals
      if len(rhs) == 4:
        # ions
        return rhs[0](loc_index,self.findindexi(rhs[1:]))
      if len(rhs) == 3:
        # neutrals
        return rhs[0](loc_index,self.findindexn(rhs[1:]))
    else:
      # electrons
      return rhs(loc_index)

  def setDoubleAtIndex(self, name, ndim,loc_index,value):
    # set wall-relevant data used by UEDGE.  This function should stay as is,
    # all implementation should be done in the functions registered in
    # the map, which are in the file uewall.py.
    # It is assumed that any invocation of setDoubleAtIndex is targeted at
    # quantities for communication with the wall, so name has a "_EW" attached.
    #
    if name.find("_EW") < 0:
      raise name+" is invalid edge-wall interface name"
   #bbb.writeToLog( "Inside uefacets::setDoubleAtIndex with name ", name, " value ", value )
    rhs =self.uewall.setMap[name]
    if type(rhs)==types.ListType:
      # ions or neutrals
      if len(rhs) == 4:
        rhs[0](value,loc_index,self.findindexi(rhs[1:]))
      if len(rhs) == 3:
        # neutrals
        rhs[0](value,loc_index,self.findindexn(rhs[1:]))
    else:
      # electrons
      rhs(value,loc_index)


  def advance(self, tnew):
    # sys.stderr.write("Uedge.advance: entered.\n")
    # tentatively advances state from current time to time + dt = tnew, and
    # does NOT return the updated values on the boundary of the quantities
    # that were not input.
    # To proceed to a next step, call ueAdvance again.
    # To retake this step, call ueReset and then this method again.
    # variables returned are those from the end of the step, in sequence:
    #  t,nivar,ngvar,upvar,tivar,tevar
    if not bbb.mype:
      # bbb.writeToLog("Uedge.advance: entered." )
      pass
    self.t_old = self.t
    self.t = tnew
    dt = self.t - self.t_old
    dt_min = 1.e-12

    bbb.dtreal = max(dt, dt_min)
    bbb.dt_tot = 0.
    bbb.t_stop = bbb.dtreal
    # bbb.iprtrundt = 1
    if (self.initcall == 1):
      if not bbb.mype:
        bbb.writeToLog("Calling bbb.exmain.")
      # sys.stderr.write("Uedge.advance: calling bbb.exmain.\n")
      bbb.exmain()
      # sys.stderr.write("Uedge.advance: bbb.exmain returned.\n")
    else:
      if not bbb.mype:
        bbb.writeToLog( "Calling bbb.rundt." )
      bbb.rundt()
    self.initcall = 0

    if not bbb.iterm==1:
      return 1
    else:
      return 0

  def interprett(self):
    # reads plasma profile data from DIII-D in the form of tanh fits for
    # ne (=ni) and te, and spline fits for ti. Data files must be called
    # netanh1.dat, tetanh1.dat, tislpline1.dat. Then calculates transport
    # coefficients dif_use, kye_use, and kyi_use (for particle D, Chi_e,
    # and Chi_i) using the uedge interpretive mode; valid only in core
    # with values outside taken as continuation of separatrix values. Leaves
    # calc transport coeff and plasma profiles updated for further simulation.
    bbb.interptrans()

  def reset(self):
    # resets Uedge variables to their state prior to last call to ueAdvance.
    # returns time at beginning of last call to ueAdvance
    bbb.nis=copy(bbb.ni0)
    bbb.ups=copy(bbb.up0)
    bbb.ngs=copy(bbb.ng0)
    bbb.tes=copy(bbb.te0)
    bbb.tis=copy(bbb.ti0)
    self.t = self.t_old
    # return self.t_old
    return

  def setShareDir(self, val):
    aph.sharedir = val

  def setNi_CE(self, val, index):
    bbb.isnicore = 1
    bbb.ncore[index] = val

  def setNg_CE(self, val, index):
    bbb.isngcore = 1
    bbb.ngcore[index] = val

  def setUp_CE(self, val, index):
    bbb.isupcore = 5
    # Facets agrees to use toroidal angular velocity; UEDGE isupcore=5
    # assumes ang. velocity is const but inut quantity is flux surface
    # av of vtor.   Connection is <vtor> = Omega*<R>, and
    # <R> = (Integral dArea/Bpol)/(Integral dArea/(RBol))
    # uedge's upcore does not.
    #bbb.upcore[index] = val*self.Raverage
    bbb.utorave = val*self.Raverage

  def setTi_CE(self, val, index):
    bbb.iflcore = 0
    # eventually may want to set an array of ion temperatures
    bbb.tcorei = val

  def setTe_CE(self, val):
    bbb.iflcore = 0
    bbb.tcoree = val

  def setTg_CE(self, val, index):
    raise "Setting of gas temperature not currently implemented in UEDGE"

  def setNiFlux_CE(self, val, index):
    bbb.isnicore = 3
    bbb.curcore[index] = val*bbb.qe*com.area_core

  def setNgFlux_CE(self, val, index):
    raise "Gas flux b.c. from core is not implemented in UEDGE"

  def setUpFlux_CE(self, val, index):
    bbb.isupcore = 4
    # The following assumes that FACETS adopts UEDGE's convention:
    # The flux used on the interface is the flux of toroidal angular momentum.
    # Unlike uedge's other flux inputs, this one is average per unit area, not
    # integrated over the flux surface
    bbb.lzflux[index] = val

  def setTiFlux_CE(self, val, index):
    bbb.iflcore = 1
    # eventually may want to set an array of ion heat fluxes
    bbb.pcorei = val * com.area_core
    if not bbb.mype:
      bbb.writeToLog( "setTiFlux_CE: val = " + `val` + ", global_area = " + `com.area_core` + ", global_flux = " + `bbb.pcorei` )

  def setTeFlux_CE(self, val):
    bbb.iflcore = 1
    bbb.pcoree = val * com.area_core
    if not bbb.mype:
      bbb.writeToLog( "setTeFlux_CE: val = " + `val` + ", global_area = " + `com.area_core` + ", global_flux = " + `bbb.pcoree` )


  def setTgFlux_CE(self, val, index):
    raise "Gas heat flux b.c. not currenetly implemented in UEDGE"

  # get boundary parameters in the UEDGE object.  Any data processing and
  # massaging that needs to be done should be performed in these
  # functions.

  def getNi_CE(self, index):
    ns_CEBdry = bbb.fluxsurfav1(bbb.nis[self.ixpt11:self.ixpt21,0,index])
    return ns_CEBdry

  def getNg_CE(self, index):
    nsgas_CEBdry = bbb.fluxsurfav1(bbb.ngs[self.ixpt11:self.ixpt21,0,index])
    return nsgas_CEBdry

  def getUp_CE(self, index):
    # convert from uedge's parallel velocity array to toroidal rotation
    # frequency.   And then do a flux surface average.
    ixpt11=self.ixpt11;ixpt21=self.ixpt21
    OmegaTor = bbb.ups[ixpt11:ixpt21,0,index]*bbb.rbfbt[ixpt11:ixpt21,0]/  \
               com.rm[ixpt11:ixpt21,0,0]
    return bbb.fluxsurfav1(OmegaTor)

  def getTi_CE(self, index):
    ti_CEBdry = bbb.fluxsurfav2(bbb.tis)/bbb.qe
    return ti_CEBdry

  def getTe_CE(self):
    te_CEBdry = bbb.fluxsurfav2(bbb.tes)/bbb.qe
    return te_CEBdry

  def getTg_CE(self, index):
    tsgas_CEBdry = bbb.fluxsurfav1(bbb.tg[self.ixpt11:self.ixpt21,0,index])
    return tsgas_CEBdry

  def getNiFlux_CE(self, index):

    nsFlux_CEBdry = sum(bbb.fniy[self.ixpt11:self.ixpt21,0,index])/   \
                    com.area_core
    return nsFlux_CEBdry

  def getNgFlux_CE(self, index):
    ngFlux_CEBdry = sum(bbb.fngy[self.ixpt11:self.ixpt21,0,index])/   \
                    com.area_core
    return ngFlux_CEBdry

  def getUpFlux_CE(self, index):
      # z angular momentum density flux
      ixpt11=self.ixpt11;ixpt21=self.ixpt21
      # fmity appears to be broken, so use fmiy for now, ignoring distinction
      # between toroidal momentum and parallel momentum
      v_parFlux_CEBdry = sum(bbb.fmiy[ixpt11:ixpt21,0,index]*com.rm[ixpt11:ixpt21,0,0])/ \
                         com.area_core
      return v_parFlux_CEBdry

  def getTiFlux_CE(self, index):
    # eventually may want to get an array of ion heat fluxes
      tsFlux_CEBdry = sum(bbb.feiy[self.ixpt11:self.ixpt21,0])/(com.area_core)
      return tsFlux_CEBdry


  def getTeFlux_CE(self):
      tsFlux_CEBdry = sum(bbb.feey[self.ixpt11:self.ixpt21,0])/(com.area_core)
      return tsFlux_CEBdry

  def getTgFlux_CE(self, index):
    # For now the neutral energy flux is just taken as the gas temperature
    # times the neutral particle flux
    return self.getNgFlux_CE(index)*self.getTg_CE(index)*joule_per_ev

  def getDensDif_CE(self, index):
    return self.gradAv_CE(bbb.fcdif*bbb.difni[index]+bbb.dif_use[:,:,index])
  def getDensConv_CE(self, index):
    return self.gradAv_CE(bbb.vy_use[:,:,index])
  def getParmomDif_CE(self, index):
    return self.gradAv_CE(bbb.fcdif*bbb.travis[index]+bbb.tra_use[:,:,index])
  def getParmomConv_CE(self, index):
    return self.gradAv_CE(bbb.vyup_use)
  def getTiDif_CE(self, index):
    return self.gradAv_CE(bbb.fcdif*bbb.kyi+bbb.kyi_use)
  def getTiConv_CE(self, index):
    return self.gradAv_CE(bbb.vyti_use)
  def getTeDif_CE(self):
    return self.gradAv_CE(bbb.fcdif*bbb.kye+bbb.kye_use)
  def getTeConv_CE(self):
    return self.gradAv_CE(bbb.vyte_use)

  # Utility routines to fetch the 2D+species arrays of the variables
  def getNi2D(self, name):
    return bbb.nis
  def getNg2D(self, name):
    return bbb.ngs
  def getUp2D(self, name):
    return bbb.ups
  def getTe2D(self, name):
    return bbb.tes
  def getTi2D(self, name):
    return bbb.tis

  """
  Dump the mesh and variables to a file.
  The type of file is determined by the suffix:
    pdb for PDB, h5 for HDF5, hdf for HDF4.
  If hdf5, the output is vizschema compliant.
  """
  def dump(self, fname):
    # bbb.writeToLog( "Entered dump with fname =", fname )

# Dump to hdf5
    if fname[len(fname) - 3:] == ".h5":
      doTransposeFlag=1
      flx.dump_vshdf5(fname, self.t, doTransposeFlag)
      bbb.writeToLog( "Dumped to hdf5" )
      return

# Dump to hdf4
    if fname[len(fname) - 4:] == ".hdf":
      dumpedobjs = Forthon.pydump(fname, hdf=1, returnfobjlist=1)
      bbb.writeToLog( "Dumped " + `dumpedobjs` + "to hdf" )
      return

# Default: dump to pdb
    dumpedobjs = Forthon.pydump(fname, hdf=0, returnfobjlist=1)
    bbb.writeToLog( "Dumped " + `dumpedobjs` + "to pdb" )

  """
  Restore the variables (not the mesh) from a file.
  The type of file is determined by the suffix:
    pdb for PDB, h5 for HDF5, hdf for HDF4.
  """
  def restore(self, fname):
    if not bbb.mype:
      # bbb.writeToLog( "Uedge.restore: entered." )
      pass
    # Before we do anything, restore from python file if specified
    pyrestart=bbb.pyrestart_file[0].strip()
    if pyrestart != "":
      if os.path.exists(pyrestart):
        bbb.writeToLog( "Including py file in restart:  " + pyrestart )
        execfile(pyrestart)
      else:
        bbb.writeToLog( "ERROR: "+ pyrestart + " does not exist." )

    try:
# Restore from hdf5
      if fname[len(fname) - 3:] == ".h5":
        # raise UedgeError("In restore.  have_tables =" + `have_tables`)
        # msg = "In restore in uefacets.py to open"
        if not bbb.mype:
          bbb.writeToLog( "Uedge.restore: Opening %s on rank %d."%(fname,bbb.mype) )
          bbb.writeToLog( "time = " + `self.t` )
        # For python, I want to do the transpose
        doTransposeFlag=1
        errval = 0
        flx.restore_vshdf5(fname, doTransposeFlag, errval)
        if ( errval != 0 ):
          return 1
        if not bbb.mype:
          pass
          # bbb.writeToLog( "Uedge.restore: flx.restore_vshdf5 returned." )

# Restore from hdf4
      elif fname[len(fname) - 4:] == ".hdf":
        bbb.writeToLog( "Restoring following variables from hdf" )
        dumpedobjs = Forthon.restore(fname, ls=1)

# Default: restore from pdb
      else:
        msg = "Restoring from pdb"
        bbb.writeToLog( msg )
        # raise UedgeError(msg)
        Forthon.restore(fname, ls=0)

# Iterate once to a solution
#       bbb.exmain()

    except string, ex:
      ex += ".  File name = fname."
      raise ex

# set bbb.restart=1 and set up the flux surface average,
# mass and charge arrays for name setting
# For hdf5 files, we handle the commonsetup.
    if not fname[len(fname) - 3:] == ".h5":
         self.commonsetup(restart=1)
    else:
         # ion atomic mass and charge state lists
         self.imasslist = bbb.minu.round(0).tolist()
         self.chargelist = bbb.ziin.round(0).tolist()
         # neutral atomic mass list, construct from unique members of imasslist.
         # this will fail (produce run-time error) if com.ngsp is bigger than
         # the length of this list; but it is also a failure of the physics
         # specification.
         self.nmasslist = uniquelist(self.imasslist)

    self.ixpt11 = com.ixpt1 + 1
    self.ixpt21 = com.ixpt2 + 1

    ixpt11 = self.ixpt11;ixpt21=self.ixpt21
    # bbb.writeToLog( "sy =", com.sy[ixpt11:ixpt21,0] )
    bbb.writeToLog( "area = "+ `com.area_core` )
    self.Raverage = bbb.fluxsurfav1(com.rm[ixpt11:ixpt21,0,0])
    # In above, making use of thin-ness of guard cell so needn't worry
    # about using centered cell value of r.

  def DoCommand(self,command):
    # Executes an arbitrary python command expressed as a string.
    # This allows skilled UEDGE users to have full access to uedge's
    # python interface via the FACETS interface
    exec(command) in __main__.__dict__

  def ListPackages(self):
    # returns a list of uedge's packages.   Variable names within
    # a package are requested as packagename.variablename
    return package()

  def ListVariables(self,packagename):
    # returns a list of the settable variables in the package whose name
    # ispackagename.
    # if packagename is an empty string then a list of settable variables in
    # all packages is returned
    if packagename != "":
      exec("vars = "+packagename+".varlist()")
      return vars
    else:
      packages = package()
      varlist = []
      for packagename in packages:
        exec("vars = "+packagename+".varlist()")
        varlist = varlist+vars
      return varlist

  def ListVarsAndVals(self,packagename):
    # returns a dictionary (pairs of variable names and values) for package
    # packagename.  In contrast to ListVariables, a packagename must be
    # specified or an exception is thrown.
    exec("varsvals = "+packagename+".getdict()")
    return varsvals

  def Dump(self,fnameroot):
    #dump variables to both HDF and PDB files, for now, until we find a good HDF reader for Python.
    # file named fname.  Files are named by appending .hdf or .pdb to fnameroot
    # For now use pydump until we figure out where uedge's dump method
    # comes from.
    pdbname = fnameroot+".pdb"
    hdfname = fnameroot+".hdf"
    pydump(hdfname,hdf=0)
    pydump(pdbname,hdf=1)

  def findindexi(self,params):
    # Returns uedge plasma species index corresponding to atomicmass and charge.
    # params is a list containing atomicnumber,atomicmass,charge
    # Requires masslist and chargelist to be set up as lists of masses and
    # charge states for each species index to be set up in advance of calling
    # For now, assuming species uniquely identified by mass and charge state.
    # Really we should allow for isotopes of different elements having same
    # atomic mass; this will come later.
    #
    # find all indices with specified mass
    # number of entries with proper mass
    atomicnumber=params[0]
    mass=params[1]
    charge=params[2]
    masslist=self.imasslist
    chargelist=self.chargelist
    numberwithmass = masslist.count(mass)
    massindices = []
    istart=0
    for i in range(numberwithmass):
        imass=masslist[istart:].index(mass)+istart
        massindices += [imass]
        istart=imass+1
    for ind in massindices:
        if chargelist[ind] == charge:
            return ind
    # if we've gotten this far, our mass, charge combo is not in list
    raise "Index error, specified mass and charge not in species list"

  def findindexn(self,params):
    # Find index for neutrals.  for now, this is just a lookup by atomic mass
    # params is a list containing atomicnumber,atomicmass
    return self.nmasslist.index(params[1])

def restore(fname):
  ueobj = Uedge()
  ueobj.restore(fname)

def dump(fname):
  ueobj = Uedge()
  ueobj.dump(fname)

def uniquelist(list1):
  # returns a list of the unique elements in list
  templist=sorted(list1)
  lenlist=len(templist)
  for i in range(lenlist-1):
    index=lenlist-i-1
    if templist[index-1]==templist[index]:
      del templist[index]
  return templist

def set_uedgeComm(communicator):
  ireturn = bbb.set_uedgeComm(communicator)




# Setup file to run time-dependently using dtreal
# Change dtreal for starting dt and savefname to change pfb file name
# Once variables are set, read rdrundt to execute a time-dependent run
from uedge import *

i_stor = 0
nfe_tot = 0
savefn = "savedt.hdf5"  # name of hdf5 savefile written every timestep

bbb.rdtphidtr = 1e20	# ratio dtphi/dtreal
bbb.ismfnkauto = 1		# if =1, mfnksol=3 for dtreal<dtmfnk3, otherwise=-3
bbb.dtmfnk3 = 5.e-4		# dtreal for mfnksol sign change if ismfnkauto=1
bbb.mult_dt = 3.4		# factor expanding dtreal after ii2max steps
bbb.ii1max = 500		# number of changes to dtreal
bbb.ii2max = 5		    # number of timesteps at current dtreal
bbb.itermxrdc = 7	    # value of itermx used by rdcontdt
bbb.incpset = 7		    # iterations until Jacobian is recomputed
bbb.ftol_dt = 1.e-5		# fnrm tolerance for the time-dependent steps
bbb.ftol_min = 1e-9		# value of fnrm where time advance will stop
bbb.dt_tot = 0.		    # tot time accumulated for run (output, not input)
bbb.t_stop = 100.		# value of dt_tot (sec) where calculation will stop
bbb.dt_max = 100.		# maximum time step for dtreal
bbb.dt_kill = 1e-14		# min allowed time step; rdcontdt stops if reached
bbb.deldt_min = 0.04	# minimum relative change allowed for model_dt > 0
bbb.initjac = 0		    # if=1, calc initial Jac upon reading rdcontdt
bbb.numrevjmax = 2		# number of dt reductions before Jac recalculated
bbb.numfwdjmax = 1		# number of dt increases before Jac recalculated
###bbb.ismmaxuc = 1		   # =1 for intern calc mmaxu; =0,set mmaxu & dont chng
bbb.irev = -1		    # flag to allow reduced dt advance after cutback
bbb.rlx = 0.9		    # max. change in variable at each linear iteration
bbb.itermx = 7	 	    # max. number of linear iterations allowed
bbb.tstor_s = 1e-5		# beginning time for storing solution
bbb.tstor_e = 1e-3		# ending time for storing solution
bbb.n_stor = 0		    # number of linearly spaced storage points
bbb.ipt = 1		   	    # index of variable; value printed at step
			            # if ipt not reset from unity, ipt=idxte(nx,iysptrx+1)




from uedge import *

def uedouble():
    com.nxleg=2*com.nxleg
    com.nxcore=2*com.nxcore
    com.nycore=2*com.nycore
    com.nysol=2*com.nysol
    if com.nxomit > 0:
        if com.geometry=="dnbot":
            com.nxomit = com.nxleg[0,0]+com.nxcore[0,0] + 2*com.nxxptxx + 1
        else:
            com.nxomit=2*(com.nxomit-2*com.nxxptxx) + 2*com.nxxpt  # assumes com.nxomit removes 1/2 SOL
    if com.nyomitmx == 1: com.nysol = 1
    if grd.kxmesh == 4:
        dxgasold=grd.dxgas
        alfxold=grd.alfx
        grd.alfx=alfxold/2.
        grd.dxgas=dxgasold*(exp(grd.alfx)-1)/(exp(alfxold)-1)
        grd.nxgas=2*grd.nxgas
    bbb.restart=1
    bbb.newgeo=1
    bbb.gengrid=1
    bbb.isnintp=1
    grd.ixdstar = com.nxcore[0,1]+1



#Coarse mesh [com.nx=8, com.ny=4] for DIII-D MHD equilibrium
#Uses diffusive neutrals, so five variables [bbb.ni,bbb.upi,Te,Ti,bbb.ng]
#
import sys,os

sys.path.insert(0,os.getcwd()+'/../../../pyscripts')
sys.path.insert(0,os.getcwd()+'/../../..')
from uedge import *

failed = False

if failed: raise Exception('data error')



#!/usr/bin/env python

# Import all required libraries

import optparse
import sys

import numpy
import pylab
import string
import tables

import ueplotdata

#
# This function tests to see if a point lies inside a given polygon
#
# @param polyX array of x point values for the polygon
# @param polyY array of y point values for the polygon
# @param x value of the x coordinate of the point to test
# @param y value of the y coordinate of the point to test
#
# @return bool specifying if point is inside the polygon
#
def polyContains(polyX, polyY, x, y):
    retval = False
    
    j = polyX.shape[0] - 1
    for i in range(0, polyX.shape[0]):
        if ( polyY[i] < y and polyY[j] >= y or
             polyY[j] < y and polyY[i] >= y ):
            if ( polyX[i] + (y-polyY[i]) / (polyY[j]-polyY[i])
                 * (polyX[j]-polyX[i]) < x ):
                retval = not retval
        j = i

    return retval

#
# Construct a 2D mapped grid interpolator
#
# @param xxarry 2d array of x coordinate values of the mapped grid
# @param yyarry 2d array of y coordinate values of the mapped grid
# @param zzvals 2d array of z values at each grid point
# @param nxarry N array of x coordinates at which to interpolate
# @param nyarry N array of y coordinates at which to interpolate
#
# @return N array of values at the interpolation points
#
def mappedInterpolate(xxarry, yyarry, zzvals, nxarry, nyarry):
    xx = nxarry
    yy = nyarry
    zz = numpy.zeros(xx.shape, numpy.float)

    oset = [(0,0), (1, 0), (1, 1), (0, 1)]

    for i in range(0, nxarry.shape[0]):

        doBreak = False
        mval = nval = 0
        for m in range(0, xxarry.shape[0]-1):
            for n in range(0, xxarry.shape[1]-1):

                polyX = numpy.array([ xxarry[m + oset[0][0], n + oset[0][1]],
                                      xxarry[m + oset[1][0], n + oset[1][1]],
                                      xxarry[m + oset[2][0], n + oset[2][1]],
                                      xxarry[m + oset[3][0], n + oset[3][1]] ])
                polyY = numpy.array([ yyarry[m + oset[0][0], n + oset[0][1]],
                                      yyarry[m + oset[1][0], n + oset[1][1]],
                                      yyarry[m + oset[2][0], n + oset[2][1]],
                                      yyarry[m + oset[3][0], n + oset[3][1]] ])

                contVal = polyContains(polyX, polyY, xx[i], yy[i])
                if ( contVal ):
                    print "Box found at ", m, n
                    mval = m
                    nval = n
                    doBreak = True
                    break

            if ( doBreak ):
                break

        if ( xxarry[mval + oset[0][0], nval + oset[0][1]] ==
             xxarry[mval + oset[1][0], nval + oset[1][1]] and
             yyarry[mval + oset[0][0], nval + oset[0][1]] ==
             yyarry[mval + oset[1][0], nval + oset[1][1]] ):

            x1 = xxarry[mval + oset[0][0], nval + oset[0][1]]
            x2 = xxarry[mval + oset[2][0], nval + oset[2][1]]
            x3 = xxarry[mval + oset[3][0], nval + oset[3][1]]

            y1 = yyarry[mval + oset[0][0], nval + oset[0][1]]
            y2 = yyarry[mval + oset[2][0], nval + oset[2][1]]
            y3 = yyarry[mval + oset[3][0], nval + oset[3][1]]

            z1 = zzvals[mval + oset[0][0], nval + oset[0][1]]
            z2 = zzvals[mval + oset[2][0], nval + oset[2][1]]
            z3 = zzvals[mval + oset[3][0], nval + oset[3][1]]

            a = -((-(y2*z1) + y3*z1 + y1*z2 - y3*z2 - y1*z3 + y2*z3) \
                / (-(x2*y1) + x3*y1 + x1*y2 - x3*y2 - x1*y3 + x2*y3))
            b = -((-(x2*z1) + x3*z1 + x1*z2 - x3*z2 - x1*z3 + x2*z3) \
                / (x2*y1 - x3*y1 - x1*y2 + x3*y2 + x1*y3 - x2*y3))
            c = -((-(x3*y2*z1) + x2*y3*z1 + x3*y1*z2 - x1*y3*z2 - x2 \
                * y1*z3 + x1*y2*z3)/(x2*y1 - x3*y1 - x1*y2 + x3*y2   \
                + x1*y3 - x2*y3))

            fn = lambda x, y: a*x + b*y + c

        elif ( xxarry[mval + oset[1][0], nval + oset[1][1]] ==
               xxarry[mval + oset[2][0], nval + oset[2][1]] and
               yyarry[mval + oset[1][0], nval + oset[1][1]] ==
               yyarry[mval + oset[2][0], nval + oset[2][1]] ):

            x1 = xxarry[mval + oset[0][0], nval + oset[0][1]]
            x2 = xxarry[mval + oset[1][0], nval + oset[1][1]]
            x3 = xxarry[mval + oset[3][0], nval + oset[3][1]]

            y1 = yyarry[mval + oset[0][0], nval + oset[0][1]]
            y2 = yyarry[mval + oset[1][0], nval + oset[1][1]]
            y3 = yyarry[mval + oset[3][0], nval + oset[3][1]]

            z1 = zzvals[mval + oset[0][0], nval + oset[0][1]]
            z2 = zzvals[mval + oset[1][0], nval + oset[1][1]]
            z3 = zzvals[mval + oset[3][0], nval + oset[3][1]]

            a = -((-(y2*z1) + y3*z1 + y1*z2 - y3*z2 - y1*z3 + y2*z3) \
                / (-(x2*y1) + x3*y1 + x1*y2 - x3*y2 - x1*y3 + x2*y3))
            b = -((-(x2*z1) + x3*z1 + x1*z2 - x3*z2 - x1*z3 + x2*z3) \
                / (x2*y1 - x3*y1 - x1*y2 + x3*y2 + x1*y3 - x2*y3))
            c = -((-(x3*y2*z1) + x2*y3*z1 + x3*y1*z2 - x1*y3*z2 - x2 \
                * y1*z3 + x1*y2*z3)/(x2*y1 - x3*y1 - x1*y2 + x3*y2   \
                + x1*y3 - x2*y3))

            fn = lambda x, y: a*x + b*y + c

        elif ( xxarry[mval + oset[2][0], nval + oset[2][1]] ==
               xxarry[mval + oset[3][0], nval + oset[3][1]] and
               yyarry[mval + oset[2][0], nval + oset[2][1]] ==
               yyarry[mval + oset[3][0], nval + oset[3][1]] ):

            x1 = xxarry[mval + oset[0][0], nval + oset[0][1]]
            x2 = xxarry[mval + oset[1][0], nval + oset[1][1]]
            x3 = xxarry[mval + oset[2][0], nval + oset[2][1]]

            y1 = yyarry[mval + oset[0][0], nval + oset[0][1]]
            y2 = yyarry[mval + oset[1][0], nval + oset[1][1]]
            y3 = yyarry[mval + oset[2][0], nval + oset[2][1]]

            z1 = zzvals[mval + oset[0][0], nval + oset[0][1]]
            z2 = zzvals[mval + oset[1][0], nval + oset[1][1]]
            z3 = zzvals[mval + oset[2][0], nval + oset[2][1]]

            a = -((-(y2*z1) + y3*z1 + y1*z2 - y3*z2 - y1*z3 + y2*z3) \
                / (-(x2*y1) + x3*y1 + x1*y2 - x3*y2 - x1*y3 + x2*y3))
            b = -((-(x2*z1) + x3*z1 + x1*z2 - x3*z2 - x1*z3 + x2*z3) \
                / (x2*y1 - x3*y1 - x1*y2 + x3*y2 + x1*y3 - x2*y3))
            c = -((-(x3*y2*z1) + x2*y3*z1 + x3*y1*z2 - x1*y3*z2 - x2 \
                * y1*z3 + x1*y2*z3)/(x2*y1 - x3*y1 - x1*y2 + x3*y2   \
                + x1*y3 - x2*y3))

            fn = lambda x, y: a*x + b*y + c

        elif ( xxarry[mval + oset[3][0], nval + oset[3][1]] ==
               xxarry[mval + oset[0][0], nval + oset[0][1]] and
               yyarry[mval + oset[3][0], nval + oset[3][1]] ==
               yyarry[mval + oset[0][0], nval + oset[0][1]] ):

            x1 = xxarry[mval + oset[1][0], nval + oset[1][1]]
            x2 = xxarry[mval + oset[2][0], nval + oset[2][1]]
            x3 = xxarry[mval + oset[3][0], nval + oset[3][1]]

            y1 = yyarry[mval + oset[1][0], nval + oset[1][1]]
            y2 = yyarry[mval + oset[2][0], nval + oset[2][1]]
            y3 = yyarry[mval + oset[3][0], nval + oset[3][1]]

            z1 = zzvals[mval + oset[1][0], nval + oset[1][1]]
            z2 = zzvals[mval + oset[2][0], nval + oset[2][1]]
            z3 = zzvals[mval + oset[3][0], nval + oset[3][1]]

            a = -((-(y2*z1) + y3*z1 + y1*z2 - y3*z2 - y1*z3 + y2*z3) \
                / (-(x2*y1) + x3*y1 + x1*y2 - x3*y2 - x1*y3 + x2*y3))
            b = -((-(x2*z1) + x3*z1 + x1*z2 - x3*z2 - x1*z3 + x2*z3) \
                / (x2*y1 - x3*y1 - x1*y2 + x3*y2 + x1*y3 - x2*y3))
            c = -((-(x3*y2*z1) + x2*y3*z1 + x3*y1*z2 - x1*y3*z2 - x2 \
                * y1*z3 + x1*y2*z3)/(x2*y1 - x3*y1 - x1*y2 + x3*y2   \
                + x1*y3 - x2*y3))

            fn = lambda x, y: a*x + b*y + c

        else:

            x1 = xxarry[mval + oset[0][0], nval + oset[0][1]]
            x2 = xxarry[mval + oset[1][0], nval + oset[1][1]]
            x3 = xxarry[mval + oset[2][0], nval + oset[2][1]]
            x4 = xxarry[mval + oset[3][0], nval + oset[3][1]]

            y1 = yyarry[mval + oset[0][0], nval + oset[0][1]]
            y2 = yyarry[mval + oset[1][0], nval + oset[1][1]]
            y3 = yyarry[mval + oset[2][0], nval + oset[2][1]]
            y4 = yyarry[mval + oset[3][0], nval + oset[3][1]]

            z1 = zzvals[mval + oset[0][0], nval + oset[0][1]]
            z2 = zzvals[mval + oset[1][0], nval + oset[1][1]]
            z3 = zzvals[mval + oset[2][0], nval + oset[2][1]]
            z4 = zzvals[mval + oset[3][0], nval + oset[3][1]]

            a = (-(x1*x4*(y1 - y4)*(y3*z2 - y2*z3)) + x3*(-(x4*(y3 - y4)*(y2*z1    \
                - y1*z2)) + x1*(y1 - y3)*(y4*z2 - y2*z4)) + x2*(x4*(y2 - y4)       \
                * (y3*z1 - y1*z3) - x3*(y2 - y3)*(y4*z1 - y1*z4) - x1*(y1 - y2)    \
                * (y4*z3 - y3*z4)))/(x2*(x3*(y2 - y3)*(y1 - y4) - x4*(y1 - y3)*(y2 \
                - y4)) + x1*(x4*(y2 - y3)*(y1 - y4) - x3*(y1 - y3)*(y2 - y4)       \
                + x2*(y1 - y2)*(y3 - y4)) + x3*x4*(y1 - y2)*(y3 - y4))
            b = (x4*y4*(y3*(z1 - z2) + y1*(z2 - z3) + y2*(-z1 + z3)) + x2*y2*(y4   \
                * (z1 - z3) + y1*(z3 - z4) + y3*(-z1 + z4)) + x3*y3*(y4*(-z1 + z2) \
                + y2*(z1 - z4) + y1*(-z2 + z4)) + x1*y1*(y4*(-z2 + z3) + y3*(z2    \
                - z4) + y2*(-z3 + z4)))/(x2*(x3*(y2 - y3)*(y1 - y4) - x4*(y1 - y3) \
                * (y2 - y4)) + x1*(x4*(y2 - y3)*(y1 - y4) - x3*(y1 - y3)*(y2 - y4) \
                + x2*(y1 - y2)*(y3 - y4)) + x3*x4*(y1 - y2)*(y3 - y4))
            c = (x1*x4*(y1 - y4)*(z2 - z3) + x3*(x4*(y3 - y4)*(z1 - z2) - x1*(y1   \
                - y3)*(z2 - z4)) + x2*(-(x4*(y2 - y4)*(z1 - z3)) + x3*(y2 - y3)    \
                * (z1 - z4) + x1*(y1 - y2)*(z3 - z4)))/(x2*(x3*(y2 - y3)*(y1 - y4) \
                - x4*(y1 - y3)*(y2 - y4)) + x1*(x4*(y2 - y3)*(y1 - y4) - x3*(y1    \
                - y3)*(y2 - y4) + x2*(y1 - y2)*(y3 - y4)) + x3*x4*(y1 - y2)*(y3    \
                - y4))
            d = (x4*(y3*(-z1 + z2) + y2*(z1 - z3) + y1*(-z2 + z3)) + x3*(y4*(z1    \
                - z2) + y1*(z2 - z4) + y2*(-z1 + z4)) + x1*(y4*(z2 - z3) + y2*(z3  \
                - z4) + y3*(-z2 + z4)) + x2*(y4*(-z1 + z3) + y3*(z1 - z4) + y1     \
                * (-z3 + z4)))/(x2*(x3*(y2 - y3)*(y1 - y4) - x4*(y1 - y3)*(y2      \
                - y4)) + x1*(x4*(y2 - y3)*(y1 - y4) - x3*(y1 - y3)*(y2 - y4) + x2  \
                * (y1 - y2)*(y3 - y4)) + x3*x4*(y1 - y2)*(y3 - y4))

            fn = lambda x, y: a + b*x + c*y + d*x*y

        zz[i] = fn(xx[i], yy[i])
        if ( not doBreak ):
            zz[i] = numpy.nan
            print "Box NOT found!"

    return zz

# get options from the command line

cpsr = optparse.OptionParser()

cpsr.add_option('-e', '--edge-file', dest='efile', action='store',
                help='Specify the edge data file to plot', default='')
cpsr.add_option('-s', '--edge-species', dest='species', action='store',
                help='Specify the edge species temperature to plot (ion, elec)',
                default='ion')

cpsr.add_option('-c', '--core-file', dest='cfile', action='store',
                help='Specify the core data file to plot', default='')

cpsr.add_option('-g', '--grid-file', dest='gfile', action='store',
                help='Specify the grid data file for core plots', default='')

cpsr.add_option('-l', '--log-plot', dest='logbool', action='store_true',
                help='Pass this flag to generate logarithmic scale plots',
                default=False)

cpsr.add_option('-f', '--file-suffix', dest='sufx', action='store',
                help='Use this as a suffix to uniquely name the outfiles',
                default='')

opts, args = cpsr.parse_args(sys.argv)

# Get all the data from the HDF5 file

fh = tables.openFile(opts.efile)

uen = fh.getNode('/bbb/uegrid')
ueg = uen.uegrid.read()

udn = fh.getNode('/bbb')
if opts.species == 'ion':
    ued = udn.tis.read() * 6.21e18
elif opts.species == 'elec':
    ued = udn.tes.read() * 6.21e18
else:
    print 'Invalid species plot requested'
    exit(1)

ifg = fh.getNode('/bbb/uegrid/blockInfo')

ysp = ifg._v_attrs.__getattr__('ySeparatrix')
xsp1 = ifg._v_attrs.__getattr__('xLeftSeparatrix')
xsp2 = ifg._v_attrs.__getattr__('xRightSeparatrix')

yoff = ifg._v_attrs.__getattr__('yMagneticAxis')
xoff = ifg._v_attrs.__getattr__('xMagneticAxis')

fh.close()

udata = ueplotdata.UePlotData(opts.efile)

# Process the private flux region

pfXX, pfYY = udata.getNodalPFGrid()
pfDA = udata.getCenteredPFData()

# Process the core layer

coXX, coYY = udata.getNodalCoreGrid()
coDA = udata.getCenteredCoreData()

# Process the scrape off layer

scXX, scYY = udata.getNodalSOLGrid()
scDA = udata.getCenteredSOLData()

# Read in all the data from the equilibrium data file

fh = open(opts.gfile, 'r')

lns = fh.readlines()[1:]
l1 = lns[0].replace(',', ' ').split()
ii = string.atoi( l1[ l1.index('i=')+1 ] )
jj = string.atoi( l1[ l1.index('j=')+1 ] ) - 5

fh.close()

# Process the core region grid

eqXX = numpy.zeros((ii, jj), numpy.float)
eqYY = numpy.zeros((ii, jj), numpy.float)
eqDA = numpy.zeros((ii, jj), numpy.float)

lns = lns[1:]
k = 0

for j in range(0, jj):
    for i in range(0, ii):
        l = lns[k].split()
        eqXX[i,j] = string.atof( l[0] )
        eqYY[i,j] = string.atof( l[1] )
        eqDA[i,j] = string.atof( l[2] )
        k = k+1

# Read in the data from the core grid fild

fh = tables.openFile(opts.cfile)

con = fh.getNode('/facets/core')
cog = con.qOld.read()
temp = cog[:,1]/(1.5*cog[:,0])*1e3

fh.close()

# Process the core region data

dx = eqDA.max() / (cog.shape[0])

eqDAt = numpy.interp(eqDA.reshape(ii*jj),
                     numpy.arange(dx / 2, eqDA.max(), dx),
                     temp)

eqDA = eqDAt.reshape(eqDA.shape)

eqDA = eqDA

# Construct the slice arrays for midplane plots

pfiXX = numpy.zeros((ueg.shape[0] - xsp2 + xsp1 - 2, ysp), numpy.float)
pfiYY = numpy.zeros((ueg.shape[0] - xsp2 + xsp1 - 2, ysp), numpy.float)

pfmXX = numpy.zeros((ueg.shape[0] - xsp2 + xsp1 - 2, ysp, 5), numpy.float)
pfmYY = numpy.zeros((ueg.shape[0] - xsp2 + xsp1 - 2, ysp, 5), numpy.float)

pfiXX[:xsp1, :] = ueg[1:xsp1+1,  1:ysp+1, 0, 0]
pfiXX[xsp1:, :] = ueg[xsp2+1:-1, 1:ysp+1, 0, 0]

pfmXX[:xsp1, :] = ueg[1:xsp1+1,  1:ysp+1, :, 0]
pfmXX[xsp1:, :] = ueg[xsp2+1:-1, 1:ysp+1, :, 0]

pfiYY[:xsp1, :] = ueg[1:xsp1+1,  1:ysp+1, 0, 1]
pfiYY[xsp1:, :] = ueg[xsp2+1:-1, 1:ysp+1, 0, 1]

pfmYY[:xsp1, :] = ueg[1:xsp1+1,  1:ysp+1, :, 1]
pfmYY[xsp1:, :] = ueg[xsp2+1:-1, 1:ysp+1, :, 1]

coiXX = numpy.zeros((xsp2 - xsp1 + 1, ysp + 1), numpy.float)
coiYY = numpy.zeros((xsp2 - xsp1 + 1, ysp + 1), numpy.float)

cocXX = numpy.zeros((xsp2 - xsp1 + 1, ysp + 1), numpy.float)
cocYY = numpy.zeros((xsp2 - xsp1 + 1, ysp + 1), numpy.float)

cocXX = numpy.zeros((xsp2 - xsp1 + 1, ysp + 1), numpy.float)
cocYY = numpy.zeros((xsp2 - xsp1 + 1, ysp + 1), numpy.float)

comXX = numpy.zeros((xsp2 - xsp1 + 1, ysp + 1, 5), numpy.float)
comYY = numpy.zeros((xsp2 - xsp1 + 1, ysp + 1, 5), numpy.float)

coiDA = numpy.zeros((xsp2 - xsp1 + 1, ysp + 1), numpy.float)

cocXX[:-1, :] = ueg[xsp1+1:xsp2+1, 0:ysp+1, 0, 0]
cocXX[-1, :] = ueg[xsp1+1, 0:ysp+1, 0, 0]

coiXX[:-1, :] = ueg[xsp1+1:xsp2+1, 0:ysp+1, 1, 0]
coiXX[-1, :] = ueg[xsp1+1, 0:ysp+1, 1, 0]

comXX[:-1, :] = ueg[xsp1+1:xsp2+1, 0:ysp+1, :, 0]
comXX[-1, :] = ueg[xsp1+1, 0:ysp+1, :, 0]

cocYY[:-1, :] = ueg[xsp1+1:xsp2+1, 0:ysp+1, 0, 1] - yoff
cocYY[-1, :] = ueg[xsp1+1, 0:ysp+1, 0, 1] - yoff

coiYY[:-1, :] = ueg[xsp1+1:xsp2+1, 0:ysp+1, 1, 1] - yoff
coiYY[-1, :] = ueg[xsp1+1, 0:ysp+1, 1, 1] - yoff

comYY[:-1, :] = ueg[xsp1+1:xsp2+1, 0:ysp+1, :, 1] - yoff
comYY[-1, :] = ueg[xsp1+1, 0:ysp+1, :, 1] - yoff

coiDA[:-1, :] = ued[xsp1+1:xsp2+1, 1:ysp+2]
coiDA[-1, :] = ued[xsp1+1, 1:ysp+2]

sciXX = numpy.zeros((ueg.shape[0] - 2, ueg.shape[1] - ysp - 2), numpy.float)
sciYY = numpy.zeros((ueg.shape[0] - 2, ueg.shape[1] - ysp - 2), numpy.float)

scmXX = numpy.zeros((ueg.shape[0] - 2, ueg.shape[1] - ysp - 2, 5), numpy.float)
scmYY = numpy.zeros((ueg.shape[0] - 2, ueg.shape[1] - ysp - 2, 5), numpy.float)

sciXX[:, :] = ueg[1:-1, ysp+1:-1, 0, 0]
sciYY[:, :] = ueg[1:-1, ysp+1:-1, 0, 1] - yoff

scmXX[:, :] = ueg[1:-1, ysp+1:-1, :, 0]
scmYY[:, :] = ueg[1:-1, ysp+1:-1, :, 1] - yoff

fh = open('uedge-coregrid.dat', 'w')

fh.write( 'i = %d, j = %d\n' % (comXX.shape[0], comYY.shape[1]) )

for j in range(0, comYY.shape[1]):
    for i in range(0, comXX.shape[0]):
        fh.write( '%e %e %e\n' % (comXX[i, j, 1], comYY[i, j, 1], coiDA[i, j]) )

fh.close()

fh = open('uedge-solgrid.dat', 'w')

fh.write( 'i = %d, j = %d\n' % (scmXX.shape[0], scmYY.shape[1]) )

for j in range(0, scmYY.shape[1]):
    for i in range(0, scmXX.shape[0]):
        fh.write( '%e %e %e\n' % (scmXX[i, j, 1], scmYY[i, j, 1], scDA[i, j]) )

fh.close()

fh = open('uedge-pflgrid.dat', 'w')

fh.write( 'i = %d, j = %d\n' % (pfmXX.shape[0], pfmYY.shape[1]) )

for j in range(0, pfmYY.shape[1]):
    for i in range(0, pfmXX.shape[0]):
        fh.write( '%e %e %e\n' % (pfmXX[i, j, 1], pfmYY[i, j, 1], pfDA[i, j]) )

fh.close()

# Plot the data

# xsp = ueg[xsp1+1, ysp+2, 1, 0]
# ysp = ueg[xsp1+1, ysp+2, 1, 1]

# xxs1 = numpy.arange(xoff, xsp + 0.0001, (xsp - xoff) / 150)
# yys1 = numpy.arange(0, ysp - yoff, (ysp - yoff) / 150)

# sln1 = mappedInterpolate(eqXX, eqYY, eqDA, xxs1, yys1)
# sln2 = mappedInterpolate(coiXX, coiYY, coDA, xxs1, yys1)

# pylab.figure(0)
# pylab.plot(numpy.sqrt((xxs1 - xoff)**2 + (yys1)**2), sln1)
# pylab.plot(numpy.sqrt((xxs1 - xoff)**2 + (yys1)**2), sln2)
# pylab.show()

fig1 = pylab.figure(1)

if ( opts.logbool ):
    ziVal = numpy.log10(numpy.min([pfDA.min(), coDA.min(), scDA.min()]))
    zaVal = numpy.log10(numpy.max([pfDA.max(), coDA.max(), scDA.max()]))
else:
    ziVal = (numpy.min([pfDA.min(), coDA.min(), scDA.min()]))
    zaVal = (numpy.max([pfDA.max(), coDA.max(), scDA.max()]))

if ( opts.logbool ):
    pylab.pcolor(pfXX, pfYY-yoff, numpy.log10(pfDA), shading='flat')
    pylab.clim(ziVal, 2*zaVal-ziVal)

    pylab.pcolor(coXX, coYY-yoff, numpy.log10(coDA), shading='flat')
    pylab.clim(ziVal, 2*zaVal-ziVal)

    pylab.pcolor(scXX, scYY-yoff, numpy.log10(scDA), shading='flat')
    pylab.clim(ziVal, 2*zaVal-ziVal)
else:
    pylab.pcolor(pfXX, pfYY-yoff, (pfDA), shading='flat')
    pylab.clim(ziVal, 2*zaVal-ziVal)

    pylab.pcolor(coXX, coYY-yoff, (coDA), shading='flat')
    pylab.clim(ziVal, 2*zaVal-ziVal)

    pylab.pcolor(scXX, scYY-yoff, (scDA), shading='flat')
    pylab.clim(ziVal, 2*zaVal-ziVal)

if ( opts.logbool ):
    iVal = numpy.log10(eqDA.min())
    aVal = numpy.log10(eqDA.max())
else:
    iVal = (eqDA.min())
    aVal = (eqDA.max())

if ( opts.logbool ):
    pylab.pcolor(eqXX, eqYY, numpy.log10(eqDA), shading='flat')
    pylab.clim(2*iVal-aVal, aVal)
else:
    pylab.pcolor(eqXX, eqYY, (eqDA), shading='flat')
    pylab.clim(2*iVal-aVal, aVal)
    
pylab.xlabel('R')
pylab.ylabel('Z')

xiVal = scXX.min()
xaVal = scXX.max()
xrVal = xaVal - xiVal

yiVal = pfYY.min() - yoff
yaVal = scYY.max() - yoff
yrVal = yaVal - yiVal

fig1.gca().set_xlim([xiVal - 0.1*xrVal, xaVal + 0.1*xrVal])
fig1.gca().set_ylim([yiVal - 0.1*yrVal, yaVal + 0.1*yrVal])
fig1.gca().set_aspect('equal')

# if ( opts.logbool ):
#     pylab.colorbar(format='1.0 x %.0f')
# else:
#     pylab.colorbar()

fig1.savefig('coupled-overplot%s.png' % opts.sufx)

fig2 = pylab.figure(2)

iVal = (ued.min())
aVal = (ued.max())

pylab.pcolor(pfXX, pfYY-yoff, (pfDA), shading='flat')
pylab.clim(iVal, aVal)

pylab.pcolor(coXX, coYY-yoff, (coDA), shading='flat')
pylab.clim(iVal, aVal)

pylab.pcolor(scXX, scYY-yoff, (scDA), shading='flat')
pylab.clim(iVal, aVal)

pylab.xlabel('R')
pylab.ylabel('Z')
pylab.title('Maximum: %g' % aVal)

fig2.gca().set_xlim([xiVal - 0.1*xrVal, xaVal + 0.1*xrVal])
fig2.gca().set_ylim([yiVal - 0.1*yrVal, yaVal + 0.1*yrVal])
fig2.gca().set_aspect('equal')

pylab.colorbar()

fig2.savefig('edge-section%s.png' % opts.sufx)

fig3 = pylab.figure(3)

iVal = (eqDA.min())
aVal = (eqDA.max())

pylab.pcolor(eqXX, eqYY, (eqDA), shading='flat')
pylab.clim(iVal, aVal)

pylab.xlabel('R')
pylab.ylabel('Z')
pylab.title('Minimum: %g' % iVal)

fig3.gca().set_xlim([xiVal - 0.1*xrVal, xaVal + 0.1*xrVal])
fig3.gca().set_ylim([yiVal - 0.1*yrVal, yaVal + 0.1*yrVal])
fig3.gca().set_aspect('equal')

pylab.colorbar()

fig3.savefig('core-section%s.png' % opts.sufx)

print "********** Core Interpolation **********"
xx1 = numpy.arange(xoff, eqXX.max()+0.0001, (eqXX.max() - xoff) / 100)
yy1 = numpy.zeros(xx1.shape, numpy.float)
sln1 = mappedInterpolate(eqXX, eqYY, eqDA, xx1, yy1)

print "********** Edge Interpolation **********"
xx2 = numpy.arange(eqXX.max()-0.02, coXX.max() + 0.0001, (coXX.max() - eqXX.max()) / 25)
yy2 = numpy.zeros(xx2.shape, numpy.float)
sln2 = mappedInterpolate(coXX, coYY, udata.getNodalCoreData(), xx2, yy2)

print "********** SOL  Interpolation **********"
xx3 = numpy.arange(coiXX.max(), sciXX.max() + 0.0001, (sciXX.max() - coiXX.max()) / 25)
yy3 = numpy.zeros(xx3.shape, numpy.float)
sln3 = mappedInterpolate(scXX, scYY, udata.getNodalSOLData(), xx3, yy3)

fig4 = pylab.figure(4)

pylab.plot(xx1, sln1)
pylab.plot(xx2, sln2)
pylab.plot(xx3, sln3)

fig4.gca().set_xlim([2.2, 2.3])
fig4.gca().set_ylim([0, 1000 ])

pylab.xlabel('Arbitrary')
pylab.ylabel('Temperature')

fig4.savefig('midplane-slice%s.png' % opts.sufx)
#pylab.show()




import sys,os
import glob
from distutils.spawn import find_executable


print "do level 1 tests of python version"


pdbdiff = find_executable('pdbdiff')
if pdbdiff == None:
    print 'No pdbdiff found in your PATH.'
    sys.exit(1)
   
tests = glob.glob('*.py')
try:
    os.mkdir('pyres')
except:
    pass
os.chdir('pyres')


results = {}
oorig = sys.stdout
eorig = sys.stderr
for tf in tests:
    if tf != 'dotest.py':
        t = os.path.splitext(tf)[0]
        try:
           with open(t+".res","wb") as f:
              sys.stdout = f
              sys.stderr = f
              execfile('../'+tf)
              f.close()
              results.update({tf:'   Passed'})
        except:
           results.update({tf:'***Failed'})
    sys.stdout = oorig
    sys.stderr = eorig

print
print
print '------------------------'
for k in results.keys():
   print  k,results[k]
  
